<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioOrderPrefixer.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/PaymentLock.php';
require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsSDK.php';

class MontonioCallbackModuleFrontController extends ModuleFrontController
{
    /**
     * @var false
     */
    private $isOrder;
    private $isOrderCartId;
    private $orderState;

    /**
     * Validate the callback from a Montonio payment method,
     * update Order state and redirect customer to end of flow.
     *
     * @return void
     */
    public function postProcess()
    {

        $orderToken = !empty(Tools::getValue('order-token')) ? Tools::getValue('order-token') : Tools::getValue('payment_token');
        $method = (string)Tools::getValue('method');

        if($orderToken) {
            $sdk = new MontonioPaymentsSDK(
                Configuration::get('MONTONIO_ACCESS_KEY'),
                Configuration::get('MONTONIO_SECRET_KEY'),
                Configuration::get('MONTONIO_ENVIRONMENT')
            );

            try {
                $tokenData = $sdk::decodePaymentToken($orderToken, Configuration::get('MONTONIO_SECRET_KEY'));
                $merchantReference = isset($tokenData->merchantReference) ? $tokenData->merchantReference : $tokenData->merchant_reference;
                $merchantRef = MontonioOrderPrefixer::removePrefix($merchantReference);
            } catch (\Exception $e) {
                PrestaShopLogger::addLog('[Montonio:invalidToken]:' . $e->getMessage());
                $this->redirectCustomerToCart();
                return;
            }
        } else {
            PrestaShopLogger::addLog('[Montonio:invalidToken]: no token');
            $this->redirectCustomerToCart();
            return;
        }

        $cartId = (int)$merchantRef;
        /**
         * Customer did not finish Montonio payment and exited the application on their own volition
         */
        if (!$method || $this->module->callPaymentFunc($method, 'didUserCancel')) {
            $this->redirectCustomerToCart();
            return;
        }

        /**
         * Perform inital validations to see if user has access to order
         */
        if (!$this->performPreValidations($cartId)) {
            $error = $this->module->l('An error occurred when validating callback from Montonio Payments.', 'callback');
            PrestaShopLogger::addLog('[Montonio:NoAccess]:' .
                $error, 3, '0000001', 'Cart/Order', $cartId);
            die(Tools::displayError($error));
        }

        $response = $this->module->callPaymentFunc($method, 'getCallbackResponse');

        if (!$response) {
            $error = $this->module->l('An error occurred when validating callback from Montonio.', 'callback');
            PrestaShopLogger::addLog('[MON:HTTPErr]:' .
                $error, 3, '0000001', 'Cart/Order', $cartId);
            die(Tools::displayError($error));
        }

        /**
         * Change order status and add payment comment
         */
        if ($response['is_paid'] && $this->isOrder) {
            if($this->orderState != '2') {
                //change payment status from waiting to accepted
                $this->changePaymentStatusFromWaitingToAccepted($this->isOrder);
            }
            $this->redirectToSupportedConfirmation($this->isOrderCartId, $this->isOrder);
            return;
        }

        /**
         * Create the order from cart
         */
        if ($response['is_paid'] && !$this->isOrder) {

            /**
             * Initialize the Cart and Customer classes
             */
            $cart         = new Cart($cartId);
            $cartCustomer = new Customer($cart->id_customer);
            $currency     = isset($response['currency']) ?
                Currency::getIdByIsoCode($response['currency'], (int) Shop::getContextShopID()) :
                null;

            /**
             * Create a lock on the cart id to prevent duplicate orders
             */
            $paymentLock = new PaymentLock();
            $lock        = null;

            if ($paymentLock->tryCreateResponseLock($cartId, $lock)) {
                try {
                    if (class_exists('Cache', false) && method_exists('Cache', 'clean')) {
                        Cache::clean('Cart::orderExists_' . (string) $cartId);
                    }

                    /**
                     * Check if order already placed
                     * This happens when a customer spams the "Return to Merchant" button
                     */
                    if ($cart->OrderExists()) {
                        $orderId = $this->getSupportedMethodOrderIdByCartId($cartId);
                        $this->redirectToSupportedConfirmation($cartId, $orderId);
                        return;
                    }

                    /**
                     * Validate the order in Prestashop by using flow variant 2 as described at
                     * https://devdocs.prestashop.com/1.7/modules/sell/techvalidation-checklist/#payment-modules-rules
                     */
                    $secureKey = $cartCustomer->secure_key;
                    $message   = null;
                    $dontRound = false;
                    $extraVal  = [
                        'transaction_id' => Tools::getValue('id_cart'),
                    ];

                    // Shipping modules' Integration ********************************************************
                    $extraData = Tools::getValue('extra_data', []);
                    foreach ($extraData as $key => $value) {
                        Context::getContext()->cookie->$key = $value;
                    }
                    // /Shipping modules' Integration *******************************************************

                    /**
                     * Prepare Payment method name for MontonioPayments
                     */
                    $methodName = $method;
                    if ($methodName == 'MontonioPayments' && isset($response['payment_method_name'])) {
                        $methodName = 'Montonio - ' . $response['payment_method_name'];
                    }

                    $this->module->validateOrder(
                        $cartId,
                        (int) Configuration::get('PS_OS_PAYMENT'),
                        $response['amount'],
                        $this->module->displayName . ' (' . $methodName . ', Cart ID: ' . $cartId . ')',
                        $message,
                        $extraVal,
                        (string) $currency,
                        $dontRound,
                        $secureKey
                    );
                } finally {
                    $paymentLock->removeResponseLock($cartId, $lock);
                }
            }

            /**
             * If this is a notification, then return 201, otherwise redirect
             */
            if ('yes' == Tools::getValue('is_notification')) {
                http_response_code(201);
                die();
            } else {
                $orderId = $this->getSupportedMethodOrderIdByCartId($cartId);
                $this->redirectToSupportedConfirmation($cartId, $orderId);
            }
        } else {
            $this->redirectCustomerToCart();
        }
    }

    /**
     * Get the payment method name
     *
     * @return void
     */
    private function getPaymentName($method)
    {
        if ($method == 'MontonioPayments') {
            return $this->module->montonioPaymentsTitle;
        }
        if ($method == 'MontonioFinancing') {
            return $this->module->montonioFinancingTitle;
        }
        if ($method == 'MontonioBlikPayments') {
            return $this->module->montonioBlikPaymentsTitle;
        }
        if ($method == 'MontonioCardPayments') {
            return $this->module->montonioCardPaymentsTitle;
        }
        return $this->module->displayName;
    }

    /**
     * Redirect customer back to cart
     */
    private function redirectCustomerToCart()
    {
        // PS 1.7+
        if (method_exists($this, 'redirectWithNotifications')) {
            $this->errors[] = $this->module->l('No payment was made.');
            $this->redirectWithNotifications($this->context->link->getPageLink('order'));
        } else {
            $this->errors[] = Tools::displayError($this->module->l('No payment was made.'));
            Tools::redirect($this->context->link->getPageLink('order', true, null, "step=3"));
        }
    }

    private function getSupportedMethodOrderIdByCartId($cartId)
    {
        if (method_exists('Order', 'getIdByCartId')) {
            return Order::getIdByCartId($cartId);
        } else {
            $orderId = Order::getOrderByCartId($cartId);
            return $orderId;
        }
    }

    private function redirectToSupportedConfirmation($cartId, $orderId)
    {

        // PS 1.7+
        if (Tools::substr(_PS_VERSION_, 0, 3) === '1.7') {
            Tools::redirect('index.php?controller=order-confirmation'
                . '&id_cart=' . $cartId
                . '&id_module=' . (int) Tools::getValue('id_module')
                . '&id_order=' . $orderId
                . '&key=' . Tools::getValue('key'));
        } else {

            $thankyouPage = Configuration::get('MONTONIO_ADVANCED_ORDER_CONFIRMATION_PAGE');

            if (!in_array($thankyouPage, array('order-detail', 'order-confirmation'))) {
                $thankyouPage = 'order-detail';
            }
            //order-detail requires auth so only offer that when authorised
            $context = Context::getContext();
            if(!$context->customer->isLogged()) {
                $thankyouPage = 'order-confirmation';
            }

            Tools::redirect($this->context->link->getPageLink($thankyouPage, null, null, array(
                'id_cart'   => $cartId,
                'id_module' => (int) Tools::getValue('id_module'),
                'id_order'  => $orderId,
                'key'       => Tools::getValue('key'),
            )));
        }
    }

    /**
     * Perform Prestashop-specific validations
     *
     * @return bool True if passed, false on validation failure
     */
    protected function performPreValidations($cartId)
    {

        /**
         * Check that Montonio is enabled
         */
        $authorized = false;
        if (Module::isInstalled('montonio')) {
            $authorized = true;
        }

        if (!$authorized || !$this->module->active) {
            return false;
        }

        /**
         * Unsupported Montonio payment method or not enabled
         */
        $method = Tools::getValue('method');
        if (!in_array($method, ['MontonioFinancing', 'MontonioPayments', 'MontonioSplit', 'MontonioBlikPayments', 'MontonioCardPayments']) ||
            !$this->module->callPaymentFunc($method, 'isEnabled')) {
            return false;
        }

        /**
         * Check cart for customer details
         */
        $cart = new Order((int) $cartId);
        $this->isOrder = $cart->id ?? false;
        $this->orderState = $cart->current_state ?? false;

        if(!$this->isOrder) {
            $cart = new Cart((int) $cartId);
        } else {
            $this->isOrderCartId =  $cart->id_cart;
        }

        if ($cart->id_customer == 0 || $cart->id_address_invoice == 0 || $cart->id_address_delivery == 0) {
            return false;
        }
        /**
         * Check that customer secure key matches cart key
         */
        $cartCustomer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($cartCustomer)) {
            return false;
        }

        if ($cartCustomer->secure_key != Tools::getValue('key')) {
            PrestaShopLogger::addLog('[MONTONIO]: Customer secure key does not match cart key');
        }

        /**
         * Cart matches customer provided in callback and can continue validation
         */
        return true;
    }

    /**
     * Change order status from waiting to accepted
     *
     * @param $orderId
     * @return void
     */
    public function changePaymentStatusFromWaitingToAccepted($orderId)
    {
        $order = new Order($orderId);
        $history = new OrderHistory();
        $history->id_order = (int)$order->id;
        $history->id_employee = 0;

        $use_existings_payment = false;
        if (!$order->hasInvoice()) {
            $use_existings_payment = true;
        }

        $history->changeIdOrderState((int)Configuration::get('PS_OS_PAYMENT'), (int)($order->id), $use_existings_payment);

        $carrier = new Carrier($order->id_carrier, $order->id_lang);
        $templateVars = array();
        if ($history->id_order_state == Configuration::get('PS_OS_SHIPPING') && $order->shipping_number) {
            $templateVars = array('{followup}' => str_replace('@', $order->shipping_number, $carrier->url));
        }

        // Save all changes
        if ($history->addWithemail(true, $templateVars)) {
            // synchronizes quantities if needed..
            if (Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT')) {
                foreach ($order->getProducts() as $product) {
                    if (StockAvailable::dependsOnStock($product['product_id'])) {
                        StockAvailable::synchronize($product['product_id'], (int)$product['id_shop']);
                    }
                }
            }
        }
    }
}