<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioFinancing.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioPayments.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioBlikPayments.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioCardPayments.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/MontonioSplit.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/PaymentLock.php';
require_once _PS_MODULE_DIR_ . 'montonio/classes/PaymentStatus.php';


class Montonio extends PaymentModule
{
    public $montonioPaymentsTitle;
    public $montonioFinancingTitle;
    public $montonioFinancingDesc;
    public $montonioSplitTitle;
    public $montonioSplitDesc;
    public $montonioCardPaymentsTitle;
    public $montonioCardPaymentsDesc;

    const MONTONIO_PAYMENTS_SUPPORTED_CURRENCIES  = ['EUR', 'PLN'];
    const MONTONIO_BLIK_SUPPORTED_CURRENCIES  = ['PLN'];
    const MONTONIO_FINANCING_SUPPORTED_CURRENCIES = ['EUR'];

    const NUM_MONTONIO_METHODS = 5;
    const MONTONIO_PAYMENT_METHODS = [
        'MontonioPayments',
        'MontonioFinancing',
        'MontonioSplit',
        'MontonioBlikPayments',
        'MontonioCardPayments'
    ];

    public $paymentTranslations;

    public $textRegionEE;
    public $textRegionLV;
    public $textRegionLT;
    public $textRegionFI;
    public $textRegionPL;
    public $textRegionDE;

    public $textCardPaymentsTitle;
    public $textCardPaymentsDescription;

    public function __construct()
    {
        $this->name             = 'montonio';
        $this->tab              = 'payments_gateways';
        $this->version          = '1.4.7';
        $this->author           = 'Montonio Finance OÜ';
        $this->is_eu_compatible = 1;
        $this->need_instance    = 0;
        $this->module_key       = '0920be4967c55d1907951c05fbbdacfc';
        $this->bootstrap        = true;

        parent::__construct();

        $this->displayName            = 'Montonio';
        $this->description            = $this->l('Montonio services for Prestashop');
        $this->confirmUninstall       = $this->l('Are you sure you want to uninstall Montonio?');

        $this->limited_currencies     = ['EUR', 'PLN'];
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        $this->moduleConfig           = [];

        $this->montonioPaymentsTitle = $this->l('Pay with your bank', 'montonio');
        $this->montonioPaymentsDesc  = $this
        ->l('Complete the purchase with a direct payment from your bank account.', 'montonio');

        $this->montonioBlikPaymentsTitle = $this->l('Pay with BLIK', 'montonio');
        $this->montonioBlikPaymentsDesc  = $this
            ->l('Complete the purchase with BLIK.', 'montonio');

        $this->montonioCardPaymentsTitle = $this->l('Card Payment', 'montonio');
        $this->montonioCardPaymentsDesc  = $this
            ->l('Pay with your credit or debit card via Montonio.', 'montonio');

        $this->montonioFinancingTitle = $this->l('Compare financing offers', 'montonio');
        $this->montonioFinancingDesc  = $this
        ->l('Select the financing offer you like best! Receive multiple financing offers with one application.', 'montonio');

        $this->montonioSplitTitle = $this->l('Pay later, no fees', 'montonio');
        $this->montonioSplitDesc  = $this
        ->l('Pay later in up to three payments without any additional fees. 0% interest, 0€ contract fee.', 'montonio');

        // Translatable strings
        $this->textRegionEE                = $this->l('Estonia', 'montonio');
        $this->textRegionLT                = $this->l('Lithuania', 'montonio');
        $this->textRegionLV                = $this->l('Latvia', 'montonio');
        $this->textRegionFI                = $this->l('Finland', 'montonio');
        $this->textRegionPL                = $this->l('Poland', 'montonio');
        $this->textRegionDE                = $this->l('Germany', 'montonio');
        $this->textCardPaymentsTitle       = $this->l('Card Payment', 'montonio');
        $this->textBlikPaymentsTitle       = $this->l('BLIK Payment', 'montonio');
        $this->textCardPaymentsDescription = $this->l('Pay with your credit or debit card via Montonio', 'montonio');

    }

    /**
     * Call user_func_array on supported payment method
     *
     * @param string $paymentMethod The payment method name in self::MONTONIO_PAYMENT_METHODS
     * @param string $functionName  The static method to be called from the $paymentMethod
     * @param mixed  ...$args        The arguments passed to the function
     *
     * @return mixed                The response from the Montonio Payment Method function
     */
    public function callPaymentFunc($paymentMethod, $functionName, ...$args)
    {
        if (!in_array($paymentMethod, self::MONTONIO_PAYMENT_METHODS)) {
            throw new Error('Unsupported payment method.');
        }

        return call_user_func($paymentMethod . '::' . $functionName, ...$args);
    }

    /**
     * Install this module and register the following Hooks:
     * [payment, displayProductPriceBlock]
     *
     * @return bool
     */
    public function install()
    {
        if (!parent::install()) {
            $this->_errors[] = $this->l('Parent install failed');
            return false;
        }

        /**
         * Register Prestashop hooks
         */
        if (!$this->registerHook('paymentOptions')
            || !$this->registerHook('displayHeader')
            || !$this->registerHook('displayProductPriceBlock')
            || !$this->registerHook('actionFrontControllerSetMedia')
        ) {
            return false;
        }

        /**
         * Update shop configuration
         */
        $shops = Shop::getShops();

        foreach (self::getDefaultConfig() as $key => $value) {
            if (Shop::isFeatureActive()) {
                foreach ($shops as $shop) {
                    if (!Configuration::updateValue($key, $value, false, null, (int) $shop['id_shop'])) {
                        return false;
                    }
                }
            } else {
                if (!Configuration::updateValue($key, $value)) {
                    return false;
                }
            }
        }

        /**
         * add order status
         */

        $status = new PaymentStatus();
        if(is_object($status)) {
            $status->addStatus();
        }

        /**
         * Default order placement mode
         */

        Configuration::updateValue('MONTONIO_PAYMENTS_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_FINANCING_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_SPLIT_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_BLIK_CREATE_ORDER', 'before');
        Configuration::updateValue('MONTONIO_CARD_CREATE_ORDER', 'before');

        return $this->installPaymentLockTable();
    }

    /**
     * Method to uninstall Montonio
     *
     * @return boolean
     */
    public function uninstall()
    {
        /**
         * Delete config values on uninstall (except for order state)
         */
        foreach (array_keys(self::getDefaultConfig()) as $key) {
            if (!Configuration::deleteByName($key)) {
                return false;
            }
        }

        return parent::uninstall();
    }

    public function installPaymentLockTable()
    {
        try {
            $paymentLock = new PaymentLock();
            $paymentLock->installResponseLockTable();
            return true;
        } catch (Exception $e) {
            $this->_errors[] = $e->getMessage();
            return false;
        }
    }

    // =========================================================================
    // Prestashop Hooks
    // =========================================================================

    /**
     * DisplayHeader hook
     *
     * @return void
     */
    public function hookDisplayHeader()
    {
        if ($this->active && $this->context->controller->php_self === 'order' ) {

            $paymentHandleStylesheet = 'grid_logos';

            if ($paymentHandleStylesheet) {
                $this->context->controller->registerStylesheet(
                    'module-' . $this->name .'-style',
                    'modules/' . $this->name . '/views/css/payment-handle/' . $paymentHandleStylesheet . '.css',
                    [
                        'media' => 'all',
                        'priority' => 200,
                    ]
                );
            }

            $this->context->controller->registerJavascript(
                'module-'.$this->name.'-lib',
                'https://public.montonio.com/assets/montonio-js/1.x/montonio.bundle.js',
                [
                    'server' => 'remote',
                    'priority' => 20,
                    'attribute' => 'async',
                ]
            );

            $this->context->controller->registerJavascript(
                'module-'.$this->name.'-local',
                'modules/' . $this->name . '/views/js/montonio-payment-handle.js',
                [
                    'priority' => 20,
                    'attribute' => 'async',
                ]
            );
        }
    }

    /**
     * Display loan calculator at product page
     *
     * @return void
     */
    public function hookDisplayProductPriceBlock($params)
    {

        if ($params['type'] !== 'price') {
            return;
        }
        if (Configuration::get('MONTONIO_FINANCING_CALC_ENABLED') !== 'yes' ||
            Configuration::get('MONTONIO_FINANCING_ENABLED') !== '1'
        ) {
            return;
        }
        if (!isset($params['product']->id)) {
            return;
        }

        /**
         * Get options for loan calculator
         */
        $calcData = [
            'data-access-key'    => Configuration::get('MONTONIO_ACCESS_KEY'),
            'data-environment'   => Configuration::get('MONTONIO_ENVIRONMENT'),
            'data-name'          => $this->montonioFinancingTitle,
            'data-css'           => Configuration::get('MONTONIO_FINANCING_CALC_CSS'),
            'data-locale'        => $this->getMontonioLocaleCode(),
            'data-logo-url'      => Configuration::get('MONTONIO_FINANCING_LOGO_URL'),
            'data-show-modal'    => Configuration::get('MONTONIO_FINANCING_CALC_SHOW_MODAL') == 'yes'
            ? 'true'
            : 'false',
            'data-info-page-url' => Configuration::get('MONTONIO_FINANCING_CALC_INFO_PAGE_URL'),
        ];

        /**
         * Add data to product page and show calculator block
         */
        $this->smarty->assign(['montonioCalculatorData' => $calcData]);
        $loanCalculatorTpl = $this->fetch('module:montonio/views/templates/hook/loan_calculator.tpl');
        return $loanCalculatorTpl;
    }

    /**
     * Return payment options available for PS 1.7+
     * Display this module as a payment option during the checkout
     *
     * @param array Hook parameters
     *
     * @return array|void
     */
    public function hookPaymentOptions($params)
    {
        /*
         * Verify if this module is active
         */
        if (!$this->active) {
            return;
        }

        $payment_options = [];

        for ($i = 1; $i <= self::NUM_MONTONIO_METHODS; $i++) {
            switch (Configuration::get('MONTONIO_METHOD_' . $i . '_TYPE')) {
                case 'MontonioBlikPayments':
                    $paymentOption = $this->callPaymentFunc(
                        'MontonioBlikPayments',
                        'getCheckoutPaymentOption',
                        $this
                    );
                    break;
                case 'MontonioPayments':
                    $paymentOption = $this->callPaymentFunc(
                        'MontonioPayments',
                        'getCheckoutPaymentOption',
                        $this
                    );
                    break;
                case 'MontonioCardPayments':
                    $paymentOption = $this->callPaymentFunc(
                        'MontonioCardPayments',
                        'getCheckoutPaymentOption',
                        $this
                    );
                    break;
                case 'MontonioFinancing':
                    $paymentOption = $this->callPaymentFunc(
                        'MontonioFinancing',
                        'getCheckoutPaymentOption',
                        $this
                    );
                    break;
                case 'MontonioSplit':
                    $paymentOption = $this->callPaymentFunc(
                        'MontonioSplit',
                        'getCheckoutPaymentOption',
                        $this,
                        MontonioSplit::SPLIT_METHOD_COMBINED
                    );
                    break;
                case 'none':
                default:
                    continue 2;
            }

            if ($paymentOption) {
                $payment_options[] = $paymentOption;
            }
        }
        return $payment_options;
    }

    /**
     * Get default configuration
     *
     * @return array
     */
    protected static function getDefaultConfig()
    {
        $config = [
            'MONTONIO_ACCESS_KEY'    => '',
            'MONTONIO_SECRET_KEY'    => '',
            'MONTONIO_ENVIRONMENT'   => '',
            'MONTONIO_ORDER_PREFIX'  => '',
            'MONTONIO_MERCHANT_NAME' => '',
        ];

        foreach (self::MONTONIO_PAYMENT_METHODS as $method) {
            $config = array_merge($config, constant($method . '::CONFIGDEFAULTS'));
        }

        for ($i = 1; $i <= self::NUM_MONTONIO_METHODS; $i++) {
            $config = array_merge($config, [
                'MONTONIO_METHOD_' . $i . '_TYPE' => 'none',
            ]);
        }

        $config['MONTONIO_METHOD_1_TYPE'] = 'MontonioPayments';
        $config['MONTONIO_METHOD_2_TYPE'] = 'MontonioFinancing';
        $config['MONTONIO_METHOD_3_TYPE'] = 'MontonioSplit1';
        $config['MONTONIO_METHOD_4_TYPE'] = 'MontonioCardPayments';
        $config['MONTONIO_METHOD_5_TYPE'] = 'MontonioBlikPayments';

        return $config;
    }

    /**
     * Use Tools::getValue on config keys for each Montonio Payment Method
     *
     * @return array
     */
    protected static function getSubmittedConfig()
    {
        $config = self::getDefaultConfig();
        array_walk($config, function (&$a, $b) {
            $a = Tools::getValue($b);
        });
        return $config;
    }

    /**
     * Use Configuration::get on config keys for each Montonio Payment Method
     *
     * @return array
     */
    protected static function getCurrentConfig()
    {
        $config = self::getDefaultConfig();
        array_walk($config, function (&$a, $b) {
            $a = Configuration::get($b);
        });
        return $config;
    }

    /**
     * Undocumented function
     *
     * @param array $settings - New Admin Settings data
     * @return string - the JSON-encoded banklist array
     */
    protected function syncBanks($settings)
    {
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsSDK.php';

        $token = MontonioPaymentsSDK::getBearerToken(
            $settings['MONTONIO_ACCESS_KEY'],
            $settings['MONTONIO_SECRET_KEY']
        );

        $env    = $settings['MONTONIO_ENVIRONMENT'];
        $apiUrl = $env === 'sandbox'
        ? 'https://api.sandbox-payments.montonio.com/pis/merchants/aspsps'
        : 'https://api.payments.montonio.com/pis/merchants/aspsps';

        $options = [
            'http' => [
                'header' => "Content-Type: application/json\r\n" .
                "Authorization: Bearer " . $token . "\r\n",
                'method' => 'GET',
            ],
        ];

        $context = stream_context_create($options);
        $result  = Tools::file_get_contents($apiUrl, false, $context);

        // error handling
        if ($result === false) {
            return false;
        } else {
            return $result;
        }
    }

    // =========================================================================
    // Admin options and Configuration
    // =========================================================================

    /**
     * Returns a string containing the HTML necessary to
     * generate a configuration screen on the admin
     *
     * @return string
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit' . $this->name)) {
            $newConfig = self::getSubmittedConfig();

            if (!$newConfig['MONTONIO_ACCESS_KEY'] || !$newConfig['MONTONIO_SECRET_KEY']) {
                $output .= $this->displayError($this->l('Invalid Configuration value'));
            } else {
                if (Tools::getValue('MONTONIO_PAYMENTS_ENABLED') == '1') {
                    if (Tools::getValue('MONTONIO_ENVIRONMENT') == 'sandbox') {
                        $newConfig['MONTONIO_PAYMENTS_CURRENT_BANKS_SANDBOX'] =
                            json_encode($this->fetchBanklist($newConfig));
                    }

                    if (Tools::getValue('MONTONIO_ENVIRONMENT') == 'production') {
                        $newConfig['MONTONIO_PAYMENTS_CURRENT_BANKS'] = json_encode($this->fetchBanklist($newConfig));
                    }
                }

                foreach ($newConfig as $key => $value) {
                    Configuration::updateValue($key, $value);
                }

                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        return $output . $this->displayForm();
    }

    /**
     * The Admin Configuration for Montonio
     *
     * @return void
     */
    public function displayForm()
    {

        /**
         * Enqueue the admin stylesheet
         */
        $this->context->controller->addCSS(
            _PS_MODULE_DIR_ . 'montonio/views/css/admin/configuration.css',
            'all'
        );

        /**
         * Get default language
         */
        $default_lang = (int) Configuration::get('PS_LANG_DEFAULT');

        /**
         * Init form fields for admin
         */
        $fields_form            = [];
        $fields_form[0]['form'] = [
            'tinymce' => true,
            'legend'  => [
                'title' => $this->l('General Settings'),
                'icon'  => 'icon-cogs',
            ],
            'input'   => [
                [
                    'type'     => 'select',
                    'label'    => $this->l('Environment'),
                    'hint'     => $this->l('Use sandbox mode for development only'),
                    'name'     => 'MONTONIO_ENVIRONMENT',
                    'required' => 'true',
                    'options'  => [
                        'query' => [
                            [
                                'id'   => 'production',
                                'name' => $this->l('Production'),
                            ],
                            [
                                'id'   => 'sandbox',
                                'name' => $this->l('Sandbox'),
                            ],
                        ],
                        'id'    => 'id',
                        'name'  => 'name',
                    ],
                ],
                [
                    'type'     => 'text',
                    'label'    => $this->l('Montonio Access Key'),
                    'name'     => 'MONTONIO_ACCESS_KEY',
                    'required' => true,
                ],
                [
                    'type'     => 'text',
                    'label'    => $this->l('Secret key'),
                    'desc'     => 'Montonio Public and Secret keys can be obtained from the integrations tab in '
                    . '<a href="https://partner.montonio.com" target="_blank">Montonio Partner System</a>'
                    . ' and Sandbox keys from <a href="https://sandbox-partner.montonio.com" target="_blank">'
                    . 'Montonio Sandbox Partner System</a>',
                    'name'     => 'MONTONIO_SECRET_KEY',
                    'required' => true,
                ],
                [
                    'type'     => 'text',
                    'label'    => $this->l('Order Prefix'),
                    'desc'     => '<strong>[MULTISTORE]</strong>If you are using Montonio with only one pair of API '
                    . 'keys, you can set the Order Prefix to distinguish between orders in the Montonio Partner System',
                    'name'     => 'MONTONIO_ORDER_PREFIX',
                    'required' => false,
                ],
                [
                    'type'     => 'text',
                    'label'    => $this->l('Merchant name'),
                    'desc'     => '<strong>[MULTISTORE]</strong>The name to be shown to customers in Montonio '
                    . 'applications. By default this is '
                    . 'your company\'s Display Name (set in the Partner System), but you can also override it here',
                    'name'     => 'MONTONIO_MERCHANT_NAME',
                    'required' => false,
                ],
            ],
            'submit'  => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ],
        ];

        foreach (self::MONTONIO_PAYMENT_METHODS as $key => $method) {
            $fields_form[]['form'] = [
                'tinymce' => true,
                'legend'  => [
                    'title' => $this->l($method),
                    'icon'  => 'icon-cogs',
                ],
                'input'   => $this->callPaymentFunc($method, 'getConfig'),
                'submit'  => [
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                ],
            ];
        }

        $paymentMethodsInput = [];
        for ($i = 1; $i < self::NUM_MONTONIO_METHODS + 1; $i++) {
            $paymentMethodsInput[] = [
                'type' => 'select',
                'label' => 'Payment method ' . $i,
                'name' => 'MONTONIO_METHOD_' . $i . '_TYPE',
                'required' => 'true',
                'options' => [
                    'query' => [
                        [
                            'id' => 'none',
                            'name' => 'Not Used',
                        ],
                        [
                            'id' => 'MontonioPayments',
                            'name' => 'Montonio Payments',
                        ],
                        [
                            'id' => 'MontonioCardPayments',
                            'name' => 'Montonio Card Payments',
                        ],
                        [
                            'id' => 'MontonioFinancing',
                            'name' => 'Montonio Financing',
                        ],
                        [
                            'id' => 'MontonioSplit',
                            'name' => 'Montonio Split Combined',
                        ],
                        [
                            'id' => 'MontonioBlikPayments',
                            'name' => 'Montonio BLIK Payments',
                        ]
                    ],
                    'id' => 'id',
                    'name' => 'name',
                ],
            ];
        }

        $fields_form[]['form'] = [
            'tinymce' => true,
            'legend'  => [
                'title' => 'Payment method order',
                'icon'  => 'icon-cogs',
            ],
            'input'   => $paymentMethodsInput,
            'submit'  => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ],
        ];

        /**
         * Setup admin options
         */
        $helper                           = new HelperForm();
        $helper->module                   = $this;
        $helper->name_controller          = $this->name;
        $helper->token                    = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex             = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->default_form_language    = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        $helper->title                    = $this->displayName;
        $helper->show_toolbar             = true;
        $helper->toolbar_scroll           = true;
        $helper->submit_action            = 'submit' . $this->name;
        $helper->toolbar_btn              = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ],
        ];

        /**
         * Fill the inputs with current data
         */
        $config = self::getCurrentConfig();
        foreach ($config as $key => $value) {
            $helper->fields_value[$key] = $value;
        }

        return $helper->generateForm($fields_form);
    }

    // =========================================================================
    // Helpers
    // =========================================================================

    /**
     * Validate currency
     *
     * @return boolean True if supported, false otherwise
     */
    public function isCurrencySupported($method)
    {
        try {
            $idCurrency = $this->context->cart->id_currency;

            $paymentCurrency = new Currency($idCurrency); // should contain iso_code
            switch ($method) {
                case 'MontonioFinancing':
                case 'MontonioSplit':
                    $supportedCurrencies = self::MONTONIO_FINANCING_SUPPORTED_CURRENCIES;
                    break;
                case 'MontonioBlikPayments':
                    $supportedCurrencies = self::MONTONIO_BLIK_SUPPORTED_CURRENCIES;
                    break;
                case 'MontonioPayments':
                case 'MontonioCardPayments':
                    $supportedCurrencies = self::MONTONIO_PAYMENTS_SUPPORTED_CURRENCIES;
                    break;
                default:
                    $supportedCurrencies = [];
            }
            return (isset($paymentCurrency->iso_code)
                && in_array($paymentCurrency->iso_code, $supportedCurrencies));
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Get Montonio Locale
     *
     * @return string
     */
    protected function getMontonioLocaleCode()
    {
        $iso   = $this->context->language->iso_code;
        $langs = [
            'lt' => 'lt',
            'et' => 'et',
            'ru' => 'ru',
            'en' => 'en',
            'pl' => 'pl',
            'de' => 'de'
        ];

        return isset($langs[$iso]) ? $langs[$iso] : 'en';
    }

    private function fetchBanklist($config)
    {
        require_once _PS_MODULE_DIR_ . 'montonio/vendor/MontonioPaymentsSDK.php';

        $result = [];

        $sdk = new MontonioPaymentsSDK(
            $config['MONTONIO_ACCESS_KEY'],
            $config['MONTONIO_SECRET_KEY'],
            $config['MONTONIO_ENVIRONMENT']
        );

        $response = $sdk->fetch_payment_methods();

        if ($response['status'] == 'SUCCESS') {
            $result = $response['data'];
        }

        return $result;
    }
}
