<?php

/**
 * We use php-jwt for JWT creation
 */
require_once 'jwt/JWT.php';

/**
 * Process order data after place order clicked
 */
class MontonioPaymentsSDK
{

    /**
     * Payment Data for Montonio Payment Token generation
     *
     * @var array
     */
    protected $_paymentData;

    protected $_accessKey;
    protected $_secretKey;
    protected $_environment;

    const MONTONIO_PAYMENTS_SANDBOX_APPLICATION_URL = 'https://sandbox-stargate.montonio.com';

    const MONTONIO_PAYMENTS_APPLICATION_URL = 'https://stargate.montonio.com';

    public function __construct($accessKey, $secretKey, $environment)
    {
        $this->_accessKey   = $accessKey;
        $this->_secretKey   = $secretKey;
        $this->_environment = $environment;
    }

    /**
     * Get the URL string where to redirect the customer to
     *
     * @return string
     */
    public function getPaymentUrl()
    {
        $base = ($this->_environment === 'sandbox')
        ? self::MONTONIO_PAYMENTS_SANDBOX_APPLICATION_URL . '/api/orders/'
        : self::MONTONIO_PAYMENTS_APPLICATION_URL . '/api/orders/';

        $data = $this->_generatePaymentToken();

        $options = [
            'http' => [
                'header'  => "Content-Type: application/json\r\n",
                'method'  => 'POST',
                'content' => json_encode(['body' => $data])
            ]
        ];

        //@TODO needs proper failure handling and error messages
        $data = $this->_apiRequest($base, $options);

        return $data['data']['paymentUrl'];
    }

    /**
     * Generate JWT from Payment Data
     *
     * @return string
     */
    protected function _generatePaymentToken()
    {
        // add expiry to payment data for JWT validation
        $this->_paymentData['accessKey'] = $this->_accessKey;
        $this->_paymentData['exp'] = time() + (10 * 60);

        return Firebase\JWT\JWT::encode($this->_paymentData, $this->_secretKey);
    }

    /**
     * Set payment data
     *
     * @param array $paymentData
     * @return MontonioPaymentsSDK
     */
    public function setPaymentData($paymentData)
    {
        $this->_paymentData = $paymentData;
        return $this;
    }

    public static function decodePaymentToken($token, $secretKey)
    {
        Firebase\JWT\JWT::$leeway = 60 * 5; // 5 minutes
        return Firebase\JWT\JWT::decode($token, $secretKey, ['HS256']);
    }

    static function getBearerToken($accessKey, $secretKey)
    {
        $data = [
            'accessKey' => $accessKey,
            'iat'        => time(),
            'exp'        => time() + (60 * 60)
        ];

        return Firebase\JWT\JWT::encode($data, $secretKey);
    }

    // =========================================================================
    // Banks list
    // =========================================================================

     /**
    * Function for making API calls with file_get_contents
    * @param string URL
    * @param Array Context Options
    * @return Array Array containing status and json_decode response
    */
    protected function _apiRequest($url, $options)
    {

        $context = stream_context_create($options);
        $result = file_get_contents($url, false, $context);

        // error handling
        if ( $result === FALSE ) {
            return [
                "status" => "ERROR",
                "data"   => false
            ];
        } else {
            return [
                "status" => "SUCCESS",
                "data"   => json_decode($result, true)
            ];
        }
    }

    public function fetch_payment_methods()
    {
        $url = $this->_environment === 'sandbox'
        ? self::MONTONIO_PAYMENTS_SANDBOX_APPLICATION_URL . '/api/stores/setup'
        : self::MONTONIO_PAYMENTS_APPLICATION_URL . '/api/stores/setup';

        $options = [
            'http' => [
                'header'  => "Content-Type: application/json\r\n" .
                "Authorization: Bearer " . MontonioPaymentsSDK::getBearerToken(
                    $this->_accessKey,
                    $this->_secretKey
                ) . "\r\n",
                'method'  => 'GET'
            ]
        ];

        return $this->_apiRequest($url, $options);
    }

}
